<?php
namespace Teknisa\CADFIS\Services;

use Teknisa\CADFIS\Factories\Ger00200EmpresaFactory;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Utilities;


class Ger00200EmpresaService{
    
    const DEV_LINK  = 'http://midia.teknisa.com/files/';
    const PROD_LINK = 'https://midia.teknisa.com/files/';
    const EXTENSION_PFX = 'pfx';
    const EXTENSION_P12 = 'p12';
    const EXTENSION_PNG = 'png';
    const EXTENSION_JPG = 'jpg';
    const EXTENSION_BMP = 'bmp';

    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $empresaFactory;

    public function __construct(Ger00200EmpresaFactory $empresaFactory, Environment $environment, \Doctrine\ORM\EntityManager $entityManager ,$dataSourceManager, $fileServerLink){
        $this->empresaFactory   = $empresaFactory;
        $this->environment      = $environment;
        $this->entityManager    = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->fileServerLink       = $fileServerLink;
    }

    public function buscar($condicao){
        return $this->dataSourceManager->findBy($condicao);
    }
    
    public function select(){
        $select = $this->empresaFactory->select();
    }
    
    public function selectparamempresa($rows){
        $select = $this->empresaFactory->selectparamempresa($rows);
    }
    
    public function insertparamempresa($cdempresa, $cdsitucrt){
        $select = $this->empresaFactory->insertparamempresa($cdempresa, $cdsitucrt);
    }
    
    public function updateparamempresa($cdempresa, $cdsitucrt){
        $select = $this->empresaFactory->updateparamempresa($cdempresa, $cdsitucrt);
    }

    public function insert(){
        $insert = $this->empresaFactory->insert();
    }
    
    public function saveInscricao ($dataSet){
        try {
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    const GET_INSCRICAO = "SELECT * FROM INSCRESTAD
                            WHERE CDEMPRESA = :CDEMPRESA
                            AND CDINSCESTA = :CDINSCESTA
                            AND NRORG = :NRORG";

    public function consultaInscricao($row){
        try {
            $params = array(
                'CDEMPRESA' => $row[0]['CDEMPRESA'],
                'CDINSCESTA' => $row[0]['CDINSCESTA'],
                'NRORG' => $this->environment->getNrOrgTrab()
            );
            return $this->entityManager->getConnection()->fetchAll(self::GET_INSCRICAO, $params);
        } catch (\Exception $e) {
            throw $e; 
        }
    }

    public  function criaDataSetInscricaoEstadual ($row){
        
        if($row[0]['__is_new']){
            if($row[0]['CDSENHACERTIF']){
                
                $rows = $this->criptografarSenhaCertificado($row);
                $dataSet = new DataSet('inscrestad', $rows);
            }else{
               
                $dataSet = new DataSet('inscrestad', $row);
            }
            return $dataSet;
        }else{

            $resultadoBanco = $this->consultaInscricao($row);
            $inscricao = $resultadoBanco;
            if(!$inscricao){
                if($row[0]['CDSENHACERTIF']){
                    $rows = $this->criptografarSenhaCertificado($row);
                    $dataSet = new DataSet('inscrestad', $rows);
                }else{
                    $dataSet = new DataSet('inscrestad', $row);
                }
            }
            else if($row[0]['CDSENHACERTIF'] !== $inscricao[0]['CDSENHACERTIF']){
                if($row[0]['CDSENHACERTIF']){
                    $rows = $this->criptografarSenhaCertificado($row);
                    $dataSet = new DataSet('inscrestad', $rows);
                }else{
                    $dataSet = new DataSet('inscrestad', $row);
                }
            }else{
                $dataSet = new DataSet('inscrestad', $row);
            }
           
            return $dataSet;
        }
    }

    public function criptografarSenhaCertificado($row) {
        try {
            // ALTERADO PARA AGUARDAR A ATUALIZAÇÃO DO DFS - A PEDIDO DA JOSIANE ALVES
            //$senhaCriptografada = base64_encode($row[0]['CDSENHACERTIF']);
            $senhaCriptografada = $this->encrypt($row[0]['CDSENHACERTIF'], 'teknisa');
            $row[0]['CDSENHACERTIF'] = $senhaCriptografada;
            
            return $row;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getInscrEstad($cdinscesta, $cdempresa, $nrorg, $getLink = false) {
       
        $inscrEsta = $this->empresaFactory->getInscrEstad($cdinscesta, $cdempresa, $nrorg);
         
        if(!empty($inscrEsta) && isset($inscrEsta[0]['DSDIRCERTIFICADO'])) {
            $inscrEsta = $inscrEsta[0];
            $inscrEsta['DSDIRCERTARRAY'] = $this->getArrayFiles($inscrEsta['DSDIRCERTIFICADO'], $getLink, 'prod');
         //   var_dump($inscrEsta.'        teste');die;
        }
        return $inscrEsta;
    }

    public function gravaCertificado ($rowCertificado){
        try {
           
            $cdinscesta = $rowCertificado['CDINSCESTA'];
            $certificados = $rowCertificado['DSDIRCERTIFICADO'];
            $empresa = $rowCertificado['CDEMPRESA'];
            $nrorg = isset($rowCertificado['NRORG']) ? $rowCertificado['NRORG'] : $this->environment->getNrorg();

            $dsDirCert    =  '';
            $arrayCert    =  array();

            foreach ($certificados as $file) {
                $fileName  = $file['name'];
                $extension = explode(".", $fileName);
                $extension = end($extension);
                
                if(($extension !== self::EXTENSION_PFX) && ($extension !== self::EXTENSION_P12)) {
                    throw \Exception($fileName.': extensão do arquivo não aceita.');
                }
                else {
                    $dsDirCert.='<'.$file['name'].'>';
                    $dsDirCert.=$file["path"];
                    $this->updateDsDirCert($cdinscesta, $empresa, $nrorg, $dsDirCert);
                 
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function encrypt($text, $salt = 'teknisa') {
        return Utilities::zeedhiEncrypt($text, $salt);
    }
    
    public function updateDsDirCert($cdinscesta, $cdempresa, $nrorg, $dsdircert) {
        $this->empresaFactory->updateDsDirCert($cdinscesta, $cdempresa, $nrorg, $dsdircert);
    }

    public function getBairro($sequential, $country, $state, $county) {
        return $this->empresaFactory->getBairro($sequential, $country, $state, $county);
    }

    public function getBairroByName($country, $state, $county, $neighborhood) {
        return $this->empresaFactory->getBairroByName($country, $state, $county, $neighborhood);
    }

    public function criarDiretorio ($inscricao){
        try {
            $raiz = realpath(__DIR__ . "/../../../../../../../../"). DIRECTORY_SEPARATOR . "anexos" . DIRECTORY_SEPARATOR . "transmissaoNFe";
            $nrorgDir = $raiz . DIRECTORY_SEPARATOR . $inscricao['NRORG'];
            $ieDir    = $nrorgDir . DIRECTORY_SEPARATOR . $inscricao['CDINSCESTA'];
            $certDir = $ieDir . DIRECTORY_SEPARATOR . "cert";
            
            if (!is_dir($certDir)) {
                if (!mkdir($certDir, 0777, true)) {
                    throw new \Exception(
                        "Não foi possivel criar o diretorio $certDir. Verifique as permissões"
                    );
                }
            }
            return $certDir;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function apagarArquivosDaPasta($dir){
        $dir = $dir.'\\';
        if(is_dir($dir)){
            if($handle = opendir($dir)){
                while(($file = readdir($handle)) !== false){
                    if($file != '.' && $file != '..'){
                        unlink($dir.$file);
                    }
                }
            }
        }
    }

    public function condicaoInscricao($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row[0]['CDEMPRESA'],
            "operator" => "=",
            "columnName" => "CDEMPRESA"
          ),
          array(
            "value" => $row[0]['CDINSCESTA'],
            "operator" => "=",
            "columnName" => "CDINSCESTA"
          )
        );
        return $conditions;
    }

    public function realizaConsultaBanco($conditions,$dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        $dataSet = $this->buscar($filterCriteria);
        return $dataSet;
    }
    
    public function getArrayFiles($filesString, $getLink = false, $mode = 'prod') {
        
        $filesArray = array();
        $array = array_filter(explode('<', $filesString));
        $count = 0;
        foreach ($array as $element) {
            $element = explode('>', $element);
            $file = array("fileName" => $element[0]);
            if($getLink) $file["path"] = ($mode === 'prod') ? $this->fileServerLink.$element[1] : $this->fileServerLink.$element[1];
            array_push($filesArray, $file);
            $count++;
        }
        
      //  var_dump($filesArray);die;
        return $filesArray;
    }
    
    public function saveLogoEmpresa ($rowDsdirlogomarca){
        try {
            $file = $rowDsdirlogomarca['DSDIRLOGOMARCA'];
            $empresa = $rowDsdirlogomarca['CDEMPRESA'];
            $nrorg =  $this->environment->getNrorg();

            if(isset($file[0])) {
                $fileName  = $file[0]['name'];
                $extension = explode(".", $fileName);
                $extension = end($extension);
               
                if(($extension !== self::EXTENSION_JPG) && ($extension !== self::EXTENSION_PNG)) {
                    throw new \Exception($fileName.': extensão do arquivo não aceita.');
                } 
               else {
                    $dsDirLogoEmp = '';
                    $dsDirLogoEmp.='<'.$file[0]['name'].'>';
                    $dsDirLogoEmp.=$file[0]["path"];
                    $this->updateDsDirLogoEmpresa( $empresa, $nrorg, $dsDirLogoEmp, $file[0]['name']);
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function saveNovoCodigoFolio ($rowNovoCodigoFolio){
        try {
            $nrorg =  $this->environment->getNrorg();
            $nrFolioAtu = str_pad($rowNovoCodigoFolio['NRFOLIOFEBOSATU'], 20, '0', STR_PAD_LEFT);
            $cdempresa = $rowNovoCodigoFolio['CDEMPRESA'];
            $existFolio = $this->empresaFactory->getFolioNovoCodigo('FOLIO'.$cdempresa,$nrorg);
            if(!empty($existFolio) && isset($existFolio[0]['NRSEQUENCIAL'])) {
                $this->empresaFactory->updateFolioNovoCodigo( 'FOLIO'.$cdempresa, $nrorg, $nrFolioAtu);
            }
            else {
                $this->empresaFactory->insertFolioNovoCodigo( 'FOLIO'.$cdempresa, $nrorg, $nrFolioAtu);
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updateDsDirLogoEmpresa( $cdempresa, $nrorg, $dsdircert, $name) {
        $this->empresaFactory->updateDsDirLogoEmpresa( $cdempresa, $nrorg, $dsdircert,$name);
    }
    
    public function validaCertificado($ie,$cnpj, $cdempresa){
           
        $dtvalidcertificweb = $this->validarCertificado($ie,$cnpj, $cdempresa);
       // var_dump($dtvalidcertificweb);die;
        $this->empresaFactory->atualizaDtCertifVencido($dtvalidcertificweb, $ie);
    }
    
    public function validarCertificado ($ie, $cnpj, $cdempresa) {
        $validade = "";
        $certificado = $this->empresaFactory->getCertificado($ie, $cnpj, $cdempresa);
       // var_dump($certificado);die;
        $certificadoEncontrado = false;
        
        foreach($certificado as $cert) {
            if($cert["DSDIRCERTIFICADO"]) {
                $certificadoEncontrado = true;
                $certify = $cert;
            }
        }
        
        if($certificadoEncontrado == false) {
            throw new \Exception("Certificado Digital não encontrado.");
        }
        
        if(empty($certify["CDSENHACERTIF"])){  
            throw new \Exception("Senha do certificado não informada.");
        }else{
            $senha = $this->decrypt($certify["CDSENHACERTIF"]);
        }
       
            
        $dados = $this->empresaFactory->buscardados($ie, $cdempresa);
        $dados['DSDIRCERTIFICADO'] = explode('>',$dados['DSDIRCERTIFICADO'])[1] ;
                       
        
        
        $certificado = $this->fileServerLink.$dados['DSDIRCERTIFICADO'];
        if(file_get_contents($certificado) === FALSE){
            throw new \Exception('Não foi possível ler o certificado, por favor confira a senha e se o certificado está válido.');
        }
        $data = file_get_contents($certificado);
       // var_dump($certificado);die;
        if (!openssl_pkcs12_read($data, $x509certdata, $senha)){
            $msg = openssl_error_string();
            var_dump($msg);
             throw new \Exception('Não foi possível ler o arquivo PKCS#12. Verifique a Senha!');
        } else {
           $CertPriv = array();
           $CertPriv = openssl_x509_parse(openssl_x509_read($x509certdata['cert']));
           $validade = date('d/m/Y H:i:s', $CertPriv['validTo_time_t']);
        }
        return $validade;
        
    }
    
    public function decrypt($text, $salt = 'teknisa') {
	    // ALTERADO PARA AGUARDAR A ATUALIZAÇÃO DO DFS - A PEDIDO DA JOSIANE ALVES
	    //$senha = base64_decode($text);
	    //return $senha;
	    
	    return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    public function excluircertificado($row) {
       $params = array(
            "CDINSCESTA"         => $row['CDINSCESTA'],
            "CDEMPRESA"          => $row['CDEMPRESA']
        );
        
        
        $this->empresaFactory->excluircertificado($params);

    }


}