<?php
namespace Teknisa\CADFIS\Services;

use Teknisa\CADFIS\Factories\Ger00210DesativacaoDaEmissaoAutomaticaFactory;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\ConnectionCustom;

require_once ('../vendor/pear/http_request2/HTTP/Request2.php');

class Ger00210DesativacaoDaEmissaoAutomaticaService{

    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $Ger00210DesativacaoDaEmissaoAutomaticaFactory;
    protected $urlIntegracaoCertisign;

    public function __construct(Ger00210DesativacaoDaEmissaoAutomaticaFactory $Ger00210DesativacaoDaEmissaoAutomaticaFactory, Environment $environment, \Doctrine\ORM\EntityManager $entityManager , $dataSourceManager, $newCode, $urlIntegracaoCertisign, ConnectionCustom $connectionCustom){
        $this->factory   = $Ger00210DesativacaoDaEmissaoAutomaticaFactory;
        $this->environment      = $environment;
        $this->entityManager    = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
        $this->nrorg = $this->environment->getNrOrgTrab();
        $this->urlIntegracaoCertisign = $urlIntegracaoCertisign;
        $this->connectionCustom       = $connectionCustom;
        
        $this->iniciarDependencias(null);
    }

    public function GenereteSequential($row){
        return $this->newCode->getCodeByTableName('DESEMIAUTONFCE', $this->nrorg, 1, 5);
    }
    
    public function GetNrorg(){
        return $this->nrorg;
    }
    
    public function buildJsontoUpload($nmDocumento, $documento){
        $base64 = $this->convertPDFinBase64($documento);

        $arrayupload = array(
            "fileName" => $nmDocumento,
            "bytes"  => $base64
        );

        $jsonUpload = json_encode($arrayupload, JSON_PRETTY_PRINT);

        return $jsonUpload;
    }
    
    function fsize($path) { 
      $fp = fopen($path,"r");
      $inf = stream_get_meta_data($fp); 
      fclose($fp); 
      foreach($inf["wrapper_data"] as $v) { 
        if (stristr($v, "content-length")) { 
          $v = explode(":", $v); 
          return trim($v[1]); 
        } 
      } 
      return 0;
    } 
    
    public function convertPDFinBase64($documento){
        try {
            // Filesize para arquivo vindo de URL - filesize()
            //$size = self::fsize($documento);
            $size = filesize($documento);
            
            $handle = fopen($documento,'r');
            $file_content = fread($handle, $size);
            fclose($handle);
            
            $encoded = base64_encode($file_content);
            
            return $encoded;
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getDesEmiAutoNfce() {
        try {
           $params = array(
                "NRORG" => $this->environment->getNrOrg()
           );
            
           $desemiautonfce = $this->factory->getDesEmiAutoNfce($params);
           
           if(!$desemiautonfce)
               throw new \Exception('Operação bloqueada. Signatário(s) não encontrado(s).');
           
           return $desemiautonfce;
        } catch(\Exception $e) {
            throw $e;
        }   
    }
    
    public function getDesEmiAutoNfceCpfOrEmail($cpf, $email) {
        try {
           $params = array(
                "NRORG"       => $this->environment->getNrOrg(),
                "NRINSJURSIG" => $cpf,
                "DSEMAIL"     => $email
           );
            
           $desemiautonfce = $this->factory->getDesEmiAutoNfceCpfOrEmail($params);
           
           if($desemiautonfce)
               throw new \Exception('Operação bloqueada. CPF e/ou E-mail já cadastrado para um Signatário.');
           
           return $desemiautonfce;
        } catch(\Exception $e) {
            throw $e;
        }   
    }
    
    public function getParamCentrAssinatura() {
        try {
           $params = array(
                "NRORG" => $this->environment->getNrOrg()
           );
            
           $paramcentrassinatura = $this->factory->getParamCentrAssinatura($params);
           
           if(!$paramcentrassinatura)
               throw new \Exception('Operação bloqueada. Parametrização centralizada de assinaturas não encontrada.');
           
           return $paramcentrassinatura;
        } catch(\Exception $e) {
            throw $e;
        }   
    }
    
    public function getRotasCentrAssinatura($baseurl) {
        try {
            $params = array(
                "NRORG"          => $this->nrorg,
                "DSCHAVEASSINAT" => 'DESEMIAUTONFCE'
            );
            
            $rotascentrassinatura = $this->factory->getRotasCentrAssinatura($params);
            
            if(!$rotascentrassinatura){
                $id = $this->newCode->getCodeByTableName('ROTAS_CENTR_ASSINATURA', $this->nrorg, 1, 10);
                
                $params = array(
                    "NRORG"          => $this->nrorg,
                    "ID"             => intval($id),
                    "DSCHAVEASSINAT" => 'DESEMIAUTONFCE',
                    "DSROTAASSINAT"  => '/callbackCertsing',
                    "DSURLBASE"      => $baseurl,
                    "CLIENT_ID"      => 'cd7e85178153by60953518945f9hakt19a50a4',
                    "CLIENT_SECRET"  => 'lib5449996c21f780fe30badd6521931a5sac9',
                    "CDOPERINCLUSAO" => $this->environment->getCdOperador(),
                    "CDOPERULTATU"   => $this->environment->getCdOperador(),
                    "NRORGINCLUSAO"  => $this->nrorg,
                    "NRORGULTATU"    => $this->nrorg
                );
            
                $rotascentrassinatura = $this->factory->insertRotasCentrAssinatura($params);
            }
           
            return $rotascentrassinatura;
        } catch(\Exception $e) {
            throw $e;
        }   
    }
    
    public function uploadToCertsing($nmDocumento, $documento){
        $return = array();
        
        // Busca Parametrização centralizada de assinaturas
        $paramcentrassinatura = $this->getParamCentrAssinatura();
        
        // Token para acesso a API
        $token = $paramcentrassinatura['DSTOKENCERTSIGN'];
        
        // Criação do Request para API
        $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/upload', \Http_Request2::METHOD_POST);
        
        // URL da API
        $url = $request->getUrl();
        
        $headers = array(
            // Request headers
            'Content-Type' => 'application/json',
            'Token'        => $token
        );

        $request->setHeader($headers);

        $parameters = array(
            // Request parameters
        );

        $url->setQueryVariables($parameters);

        $request->setMethod(\Http_Request2::METHOD_POST);

        $body = $this->buildJsontoUpload($nmDocumento, $documento);

        // Request body
        $request->setBody($body);
        
        $return['request']['parameters'] = $parameters;
        $return['request']['body'] = json_decode($body, true);

        try {
            $response = $request->send();
            $retorno = $response->getBody();
            $retorno = json_decode($retorno, true);
            
            $return['response'] = $retorno;
            
            return $return;
        }
        catch (\HTTP_Request2_Exception $ex) {
            $return['response'] = $ex;
            return $return;
        }
    }
    
    public function createSignatureCertsing($uploadId, $nmDocumento){
        
        // Busca Parametrização centralizada de assinaturas
        $paramcentrassinatura = $this->getParamCentrAssinatura();
        
        // Token para acesso a API
        $token = $paramcentrassinatura['DSTOKENCERTSIGN'];
        
        // Configuração do Sender/Singers/Padrao/Documento
        $sender = array(
            "NOME"  => $paramcentrassinatura['NMSENDERCERTSIGN'],
            "EMAIL" => $paramcentrassinatura['DSEMAILSENDERCERTSIGN'],
            "CPF"   => $paramcentrassinatura['NRCPFSENDERCERTSIGN']
        );
        
        $signers = array();
        
        // Busca Parametrização centralizada de assinaturas
        $signatarios = $this->getDesEmiAutoNfce();
        
        $eletrSigners = array();
        $dsemailcopia = '';
        
        foreach ($signatarios as $key => $signatario) {
            $eletrSigners[$key] = array(
                "POSICAO" => $signatario['NRSEQDESEMIAUTO'],
                "TIPO"    => 'Signer',
                "NOME"    => $signatario['NMSIGNATARIO'],
                "EMAIL"   => $signatario['DSEMAIL'],
                "CPF"     => $signatario['NRINSJURSIG']
            );
            
            if($signatario['DSEMAILCOPIA'])
                $dsemailcopia .= $signatario['DSEMAILCOPIA'];
                if(substr($signatario['DSEMAILCOPIA'], -1) != ';')
                    $dsemailcopia .= ';';
        };
        
        $padraoAssinatura = $paramcentrassinatura['IDPADRAOASSINACERTSIGN'];
        $tipoDoc          = $paramcentrassinatura['NRTIPODOCCERTSIGN'];
        
        // Email para Teste
        //$emailOperResp = 'victor.f.rezende@gmail.com;'.$dsemailcopia;
        // Email para Produção
        $emailOperResp = 'amalia@teknisa.com;contratos@teknisa.com;'.$dsemailcopia;
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/create', \Http_Request2::METHOD_POST);

            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
    
            $url->setQueryVariables($parameters);
    
            $request->setMethod(\Http_Request2::METHOD_POST);
    
            $body = $this->buildJsontoCreate($uploadId, $nmDocumento, $sender, $signers, $eletrSigners, $padraoAssinatura, $tipoDoc, $emailOperResp);
            $body = str_replace('\/', '/', $body);
            
            $return['request']['parameters'] = $parameters;
            $return['request']['body'] = json_decode($body, true);
            // Request body
            $request->setBody($body);
             
            try {
                $response = $request->send();
                
                $retorno = $response->getBody();
                $retorno = json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HTTP_Request2_Exception $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }

        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buildJsontoCreate($uploadId, $nmDocumento, $sender, $signers, $eletrSigners, $padraoAssinatura, $tipoDoc, $emailOperResp){
        /* -- Documento -- */ 
        $document = array(
            "name" => $nmDocumento,
            "upload" => array("id" => $uploadId,
                              "name" => $nmDocumento)
            );

        /* -- responsável pelo envio -- */
        $sender = array(
            "name"  => $sender['NOME'],
            "email" => $sender['EMAIL'],
            "individualIdentificationCode" => $sender['CPF']
        );
        
        /* -- Signatários Digitais -- */
       
        $signers_aux = array();

        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
            $eletrSigners_aux[$ind] = array(
                "step"  => $ind,
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }
        
        // Cria e envia o body do documento
        $arrayCreate = array(
            "document"          => $document,
            "sender"            => $sender,
            "signers"           => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );
        
        // Padrão de Assinatura - 2
        if($padraoAssinatura){
            $arrayCreate["signatureStandard"] = $padraoAssinatura;
        }
        
        // Tipo de Documento - 2889(Homologação)
        if($tipoDoc){
            $arrayCreate["typeId"] =  $tipoDoc;
        }
        
        // Email do Operador Responsável
        if($emailOperResp){
            $arrayCreate["recipients"] = array(
                "emails" => $emailOperResp,
                "attached" => true
            );
        }
        
        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);
        
        return $jsonCreate;
    }

    public function insertCentralizaAssinatura($iddocassinat, $cdempresa, $baseurl){
        
        // Busca Rotas Centraliza Assinatura
        $this->getRotasCentrAssinatura($baseurl);
        
        $id = $this->newCode->getCodeByTableName('CENTRALIZA_ASSINATURA', $this->nrorg, 1, 10);
        
        $params = array(
            "ID"             => intval($id),
            "NRORG"          => $this->nrorg,
            "DSCHAVEASSINAT" => 'DESEMIAUTONFCE',
            "IDDOCASSINAT"   => $iddocassinat,
            "CDEMPRESA"      => $cdempresa,
            "CDOPERINCLUSAO" => $this->environment->getCdOperador(),
            "CDOPERULTATU"   => $this->environment->getCdOperador(),
            "NRORGINCLUSAO"  => $this->nrorg,
            "NRORGULTATU"    => $this->nrorg
        );
            
        $this->factory->insertCentralizaAssinatura($params);
    }
    
    private function setConnection($nrorg) {
        $novoEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
        if ($novoEntityManager) {
            $this->entityManager = $novoEntityManager;
            $this->connection    = $novoEntityManager->getConnection();
        }
        else {
            $this->connection = $this->entityManager->getConnection();
        }
    }
    
    private function iniciarDependencias($nrorg) {
        if (is_null($nrorg))
            $nrorg = $this->environment->getNrOrg();
            
        $this->setConnection($nrorg); // Verificar o que é isso
        $this->factoryCallback = new Ger00210DesativacaoDaEmissaoAutomaticaFactory($this->entityManager);
        $this->newCodeCallback = new NewCode($this->connection, "NOVO_CODIGO_SC");
    }
    
    public function callbackCertsing($filedata, $nrorg) {
        $this->iniciarDependencias($nrorg);
        $idaceitarecusa = 'R';
        
        try {
            $action = $filedata['action'];
            
            if ($action == 'FLOW') {
                $documentId = $filedata['documentId'];
                $identifier = isset($filedata['identifier'])?$filedata['identifier']:null;
                
                $params = array(
                    "NRORG"          => $nrorg,
                    "IDDOCASSINAT"   => $documentId,
                    "DSCHAVEASSINAT" => "DESEMIAUTONFCE"
                );
                
                $centralizaAssinatura = $this->factoryCallback->getCentralizaAssinatura($params);
                
                if ($centralizaAssinatura) {
                    $idcentrassina = $centralizaAssinatura['ID'];
                    
                    if($centralizaAssinatura['IDSTATASSINAT'] != 'R'){
                        $idaceitarecusa = 'A';
                        
                        $params = array(
                            "NRORG"       => $nrorg,
                            "CDEMPRESA"   => $centralizaAssinatura['CDEMPRESA'],
                            "IDIMPCUPMEI" => "N"
                        );
                        
                        // Busca Empresa - Desativa Obrigatoriedade da Emissão do Cupom Fiscal
                        $this->factoryCallback->updateEmpresa($params);
                        
                        $params = array(
                            "NRORG"         => $nrorg,
                            "ID"            => $centralizaAssinatura['ID'],
                            "IDSTATASSINAT" => $idaceitarecusa
                        );
                        
                        // Busca Centraliza Assinatura - Altera o documento para A - APROVADO
                        $this->factoryCallback->updateCentralizaAssinatura($params);
                    }else throw new \Exception("Documento com o ID informado foi rejeitado por um ou mais signatários.");
                }else throw new \Exception("Não foi encontrado o documento com o ID informado.");
            }elseif ($action == 'SIGNATURE-NOK') {
                $documentId = $filedata['documentId'];
                $identifier = isset($filedata['identifier'])?$filedata['identifier']:null;
                
                $params = array(
                    "NRORG"          => $nrorg,
                    "IDDOCASSINAT"   => $documentId,
                    "DSCHAVEASSINAT" => "DESEMIAUTONFCE"
                );
                
                $centralizaAssinatura = $this->factoryCallback->getCentralizaAssinatura($params);
                
                if ($centralizaAssinatura) {
                    $idcentrassina = $centralizaAssinatura['ID'];
                    
                    $params = array(
                        "NRORG"         => $nrorg,
                        "ID"            => $centralizaAssinatura['ID'],
                        "IDSTATASSINAT" => $idaceitarecusa
                    );
                    
                    // Busca Centraliza Assinatura - Altera o documento para R - RECUSADO
                    $this->factoryCallback->updateCentralizaAssinatura($params);
                    
                    throw new \Exception("Documento com o ID informado foi rejeitado pelo signatário.");
                }else throw new \Exception("Não foi encontrado o documento com o ID informado.");
            }
        }catch(\Exception $e) {
            $id = $this->newCode->getCodeByTableName('LOG_CENTR_ASSINATURA', $this->nrorg, 1, 10, true);
            
            $iddocassinat    = isset($documentId)    ? $documentId    : null;
            $nrcpfsignatario = isset($identifier)    ? $identifier    : null;
            $idcentrassina   = isset($idcentrassina) ? $idcentrassina : null;
            
            $dserro = "Erro ao atualizar a situação do documento: " . $e->getMessage();
            
            $params = array(
                "ID"              => $id,
                "ID_CENTR_ASSINA" => $idcentrassina,
                "IDDOCASSINAT"    => $iddocassinat,
                "NRCPFSIGNATARIO" => $nrcpfsignatario,
                "IDACEITARECUSA"  => $idaceitarecusa,
                "DSLOG"           => $dserro,
                "NRORG"           => $nrorg
            );
            
            $this->factoryCallback->insertLogCentrAssinatura($params);
            
            throw $e;
        }
    }
}