<?php
namespace Teknisa\CADFIS\Services;

use Doctrine\DBAL\Connection;
use Teknisa\Libs\Util\Environment;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\CADFIS\Services\GeneralService as GeneralService;

class MarcProdService
{
    protected $dataSourceName = "marcprod";
    protected $entityManager;
    protected $crudService;
    
    public function __construct($entityManager, $crudService, Connection $connection, Environment $environment)
    {
        $this->entityManager = $entityManager;
        $this->crudService = $crudService;
        $this->connection = $connection;
        $this->environment = $environment;
        $this->generalService = new GeneralService($this->connection, $this->environment);
    }
    
    public function verifyMarcaExists($row)
    {
        return $this->generalService->verifyMarcaExists($row);
    }
    
    public function verifyCdProdutoExists($row)
    {
        return $this->generalService->verifyCdProdutoExists($row);
    }
    
    public function verifyMarcProdExists($row)
    {
        $query = "SELECT MARCPROD.CDMARCA, MARCPROD.CDPRODUTO, MARCPROD.NRORG 
                    FROM PRODUTO JOIN MARCPROD ON PRODUTO.CDPRODUTO = MARCPROD.CDPRODUTO 
                                              AND PRODUTO.NRORG = MARCPROD.NRORG 
                   WHERE MARCPROD.CDMARCA = :P_CDMARCA
                     AND MARCPROD.NRORG = :P_NRORG
                     AND :P_CDARVPROD LIKE PRODUTO.CDARVPROD||'%'";
        $params = array(
            'P_CDMARCA'     => $row['CDMARCA'],
            'P_CDARVPROD'   => $row['CDARVPROD'],
            'P_NRORG'       => $this->environment->getNrorg(),
        );

        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        
        return $stmt->fetch();
    }
    
    private function insertMarcProdComp($rows) {
        foreach ($rows as $row) {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'INSERE_PRODFORNCMP_MARCFORNCMP');
            $procedure->addParam(new Param('P_CDFORNECED',  Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',     Param::PARAM_INPUT, $row["CDMARCA"],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',   Param::PARAM_INPUT, $row["CDPRODUTO"],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDARVPROD',   Param::PARAM_INPUT, $row["CDARVPROD"],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ARRED',       Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNORIG',  Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
            $procedure->execute();
        }
    }
    
    public function saveImport($rows)
    {
        $this->crudService->save($this->dataSourceName, $rows);
        self::insertMarcProdComp($rows);
    }
}