<?php
namespace Teknisa\CADFIS\Services;

use Doctrine\DBAL\Connection;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Libs\Util\Environment;
use Teknisa\CADFIS\Services\GeneralService as GeneralService;

class ProdFornService
{
    protected $dataSourceName = "prodforn";
    protected $entityManager;
    protected $crudService;
    protected $generalService;
    
    public function __construct($entityManager, $crudService, Connection $connection, Environment $environment)
    {
        $this->entityManager = $entityManager;
        $this->crudService = $crudService;
        $this->connection = $connection;
        $this->environment = $environment;
        $this->generalService = new GeneralService($this->connection, $this->environment);
    }
    
    public function verifyFornecedorExists($row)
    {
        return $this->generalService->verifyFornecedorExists($row);
    }
    
    public function verifyCdProdutoExists($row)
    {
        return $this->generalService->verifyCdProdutoExists($row);
    }
    
    public function verifyUnidadeExists($row)
    {
        return $this->generalService->verifyUnidadeExists($row);
    }
    
    public function verifyProdFornExists($row)
    {
        $query = "SELECT PRODFORN.CDPRODUTO 
                    FROM PRODFORN JOIN PRODUTO ON PRODFORN.CDPRODUTO = PRODUTO.CDPRODUTO
                                              AND PRODFORN.NRORG = PRODUTO.NRORG 
                   WHERE PRODFORN.CDFORNECED = :P_CDFORNECED
                     AND ((PRODUTO.CDARVPROD LIKE :P_CDARVPROD || '%') OR (INSTR(:P_CDARVPROD, PRODUTO.CDARVPROD) = 1))
                     AND PRODFORN.NRORG = :P_NRORG";
        $params = array(
            'P_CDFORNECED'  => $row['IDTPIJURFORN'].$row['NRINSJURFORN'],
            'P_CDARVPROD'   => $row['CDARVPROD'],
            'P_NRORG'       => $this->environment->getNrorg(),
        );

        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        
        return $stmt->fetch();
    }
    
    private function insertProdFornComp($rows) {
        foreach ($rows as $row) {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'INSERE_PRODFORNCMP_MARCFORNCMP ');
            $procedure->addParam(new Param('P_CDFORNECED',      Param::PARAM_INPUT, $row['CDFORNECED'],         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',         Param::PARAM_INPUT, NULL,                       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',       Param::PARAM_INPUT, $row["CDPRODUTO"],          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDARVPROD',       Param::PARAM_INPUT, $row["CDARVPROD"],          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ARRED',           Param::PARAM_INPUT, 'P',                        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNORIG',      Param::PARAM_INPUT, NULL,                       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRARREDEMPE',     Param::PARAM_INPUT, $row["VRARREDEMPE"],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRARREDCMPFO',    Param::PARAM_INPUT, $row["VRARREDCMPFO"],       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPERPIPRFO',     Param::PARAM_INPUT, $row["VRPERPIPRFO"],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPERVARPREPRFO', Param::PARAM_INPUT, $row["VRPERVARPREPRFO"],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRDIASVALPRFORN', Param::PARAM_INPUT, $row["NRDIASVALPRFORN"],    Param::PARAM_TYPE_STR));
            $procedure->execute();
        }
    }
    
    public function saveImport($rows)
    {
        $this->crudService->save($this->dataSourceName, $rows);
        self::insertProdFornComp($rows);
    }
}