<?php

namespace Teknisa\CADFIS\Validador;

class Amazonas
{

    /**
     * Verifica se a inscrição estadual é válida para o Amazonas (AM)
     * seguindo a regra: http://www.sintegra.gov.br/Cad_Estados/cad_AM.html
     *
     * @param $inscricao_estadual string Inscrição Estadual que deseja validar.
     * @return bool true caso a inscrição estadual seja válida para esse estado, false caso contrário.
     */
    public static function check($inscricao_estadual)
    {
        $valid = true;
        // se não tiver 9 digitos não é valido
        
        if(strlen($inscricao_estadual) < 9){
            $inscricao_estadual = str_pad($inscricao_estadual, 9, 0, STR_PAD_LEFT);
        }
        if (strlen($inscricao_estadual) != 9) {
            $valid = false;
        }
        if ($valid && !self::calculaDigito($inscricao_estadual)) {
            $valid = false;
        }
        $return = array($valid, $inscricao_estadual);
        return $return;

    }

    /**
     * Valida o dígito da inscrição estadual
     *
     * Pesos: 9 8 7 6 5 4 3 2 para calculo do dígito
     * @param $inscricao_estadual string inscricao estadual
     * @return bool true caso o digito seja verificado, false caso contrário.
     */
    private static function calculaDigito($inscricao_estadual)
    {

        $soma = 0;
        $length = strlen($inscricao_estadual);
        $posicao = $length - 1;
        $peso = $length;
        $corpo = substr($inscricao_estadual, 0, $posicao);
        foreach (str_split($corpo) as $item) {
            $soma += $item * $peso;
            $peso--;
        }
        if ($soma < 11) {
            $dig = 11 - $soma;
        } else {
            $resto = $soma % 11;
            $dig = 11 - $resto;

            if ($dig >= 10) {
                $dig = 0;
            }
        }
        return $dig == $inscricao_estadual[$posicao];
    }
}