
// FILE: src/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-scheduler';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}

// FILE: src/services/ZhCardsEvents.js
ZeedhiServices.service('ZhCardsEvents', function(){
    
    var registeredEvents;
    
    (function(){
        registeredEvents = {};
    })();
    
    this.registerFunctionForClick = function(eventFunctionName){
        registerFunction('onClick', eventFunctionName);    
    };
    
    function registerFunction(eventType, eventFunctionName){
        if(!registeredEvents.hasOwnProperty(eventType)){
            registeredEvents[eventType] = [];
        }
        registeredEvents[eventType].push(eventFunctionName);
    }
    
    this.getRegisteredClickFunctions = function(){
        return registeredEvents.onClick;  
    };
        
});

// FILE: src/services/ZhIconCalendarEvents.js
ZeedhiServices.service('ZhIconCalendarEvents', function(){
    
    // registers event
    // get all registered events
    
    var calendarEvents;
    
    (function(){
        calendarEvents = { 
            'onDateClick': [ 'dateClick' ],
            'onMonthClick': [ 'monthClick' ],
            'onYearClick': [ 'yearClick' ]
        };
    })();
    
    this.registerFunctionForDayClick = function(eventFunctionName){
        registerFunction('onDateClick', eventFunctionName);    
    };
    
    this.registerFunctionForMonthClick = function(eventFunctionName){
        registerFunction('onMonthClick', eventFunctionName);
    };
    
    this.registerFunctionForYearClick = function(eventFunctionName){
        registerFunction('onYearClick', eventFunctionName);
    };
    
    function registerFunction(eventType, eventFunctionName){
        if(!calendarEvents.hasOwnProperty(eventType)){
            calendarEvents[eventType] = [];
        }
        calendarEvents[eventType].push(eventFunctionName);
    }
    
    this.getRegisteredFunctionsForDateClick = function(){
        return calendarEvents.onDateClick;  
    };
    
    this.getRegisteredFunctionsForMonthClick = function(){
        return calendarEvents.onMonthClick;
    };
    
    this.getRegisteredFunctionsForYearClick = function(){
        return calendarEvents.onYearClick;
    };
        
});

// FILE: src/services/ZhScheduler.js
function ZhSchedulerService(eventEngine, ZhIconCalendarEvents, ZhCardsEvents){
    
    (function initialize(){
        registerWidgetOnDayClickEvent();
        registerWidgetOnEventClickEvent();
    })();
    
    function registerWidgetOnDayClickEvent(){
        eventEngine.registerEvent('WidgetOnDayClick', 'schedulerCalendarDayClick');
        ZhIconCalendarEvents.registerFunctionForDayClick('schedulerCalendarDayClick');
    }
    
    function registerWidgetOnEventClickEvent(){
        eventEngine.registerEvent('WidgetOnEventClick', 'schedulerCardClick');
        ZhCardsEvents.registerFunctionForClick('schedulerCardClick');
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register("ZhSchedulerService", ZhSchedulerService);
});

// FILE: src/controllers/ZhCards.js
var CardsController = function($scope, ZhCardsEvents){
    
    /**
     * Required properties are:
     * - title
     * - description
     * 
     * Optional properties are:
     * - color
     **/
    
    var widget;
    
    (function initialize(){
        widget = $scope.widget;    
    })();
    
    /********************************************
     * Internal functions
     ********************************************/
    function updateCurrentRow(newValue){
        widget.currentRow = newValue;
    }
    
    function callRegisteredClickEvents(){
        var registeredClickEvents = ZhCardsEvents.getRegisteredClickFunctions();
        registeredClickEvents.forEach(function(functionName){
            if(widget[functionName]){
                widget[functionName]();
            }
        });
    }
    
    /********************************************
     * Functions used by template
     ********************************************/
    this.buildBorderColor = function(card){
        var border = ( widget.color && card[widget.color] ) ? '5px solid ' + card[widget.color] : 'none';
        return { 'border-left': border };
    };
    
    this.getFields = function(){
        return widget.fields;
    };
    
    this.isFieldTitleType = function(field){
        return field.type === "title";
    };
    
    this.isFieldNormalType = function(field){
        return field.type !== "title";
    };
    
    this.getFieldContent = function(card, field){
        return card[field.name];
    };
    
    this.onClick = function(clickedCard){
        updateCurrentRow(clickedCard);
        callRegisteredClickEvents(); 
    };
    
    this.isThereResult = function(){
        return widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0;
    };
    
    this.getNoResultText = function(){
        return widget.noMatchText || "No results were found.";
    };
};

ZeedhiController.controller('CardsController', CardsController);

// FILE: src/controllers/ZhIconCalendar.js
var ZhIconCalendarController = function($scope, eventAggregator, ZhIconCalendarEvents){

    var CONTROL_FORMAT_MONTH = 'YYYY-MM';
    var CONTROL_FORMAT_YEAR = 'YYYY';

    var calendarWidget;
    var lastDayClicked;

    (function(){
        calendarWidget = $scope.widget;
    })();


    /**
     * Day codes
     **/
    this.onDateClick = function(clickedDay){
        if(isDayBetweenMinAndMaxDate(clickedDay)){
            if(isDayInCurrentMonth(clickedDay)){
                if(calendarWidget.field){
                    updateCurrentDateOnField($scope.currentDate);
                }
                updateCalendarCurrentRow(clickedDay.date);
                changeVisuallySelectedDay(clickedDay);
            }
            else {
                var totalOfMonthsToChange = isDayInLastMonth(clickedDay) ? -1 : +1;
                changeCurrentMonthShownOnCalendar(totalOfMonthsToChange);
            }
        }

        callOnDateClickEvents();
    };

    function callOnDateClickEvents(){
        var dayClickEventFunctions = ZhIconCalendarEvents.getRegisteredFunctionsForDateClick();
        callEventFunctions(dayClickEventFunctions);
    }

    function isDayInCurrentMonth(day){
        return (!day.lastMonth && !day.nextMonth);
    }

    function isDayBetweenMinAndMaxDate(day){
		var date = moment(day.date, $scope.dateFormat);
		return (isDateAfterMinAllowed(date) && isDateBeforeMaxAllowed(date));
	}

	function updateCurrentDateOnField(date){
        date = date || $scope.currentDate;
        calendarWidget.field.currentDate = date;
    }

    function updateCalendarCurrentRow(newValue){
        calendarWidget.currentRow = newValue;
    }

    function changeVisuallySelectedDay(clickedDay){
        lastDayClicked = clickedDay;
    }


	/**
     * Month codes
     **/
	this.onMonthClick = function(clickedMonth){
	    callOnMonthClickEvents();
        if(isMonthBetweenMinAndMaxDate(clickedMonth)){
            $scope.currentDate = moment(clickedMonth.date, CONTROL_FORMAT_MONTH);
            $scope.changeCalendarMode('day', true);
        }
    };

    function callOnMonthClickEvents(){
        var monthClickEventFunctions = ZhIconCalendarEvents.getRegisteredFunctionsForMonthClick();
        callEventFunctions(monthClickEventFunctions);
    }

	function isMonthBetweenMinAndMaxDate(month){
        var date = month.date;
        var beginMonth = moment(date, CONTROL_FORMAT_MONTH).startOf('month');
        var endMonth = moment(date, CONTROL_FORMAT_MONTH).endOf('month');
        return (isDateAfterMinAllowed(beginMonth) && isDateBeforeMaxAllowed(endMonth));
	}


	/**
     * Year codes
     **/
    this.onYearClick = function(clickedYear){
        callOnYearClickEvents();

        if(isYearInCurrentDecade(clickedYear) && isYearBetweenMinAndMaxDate(clickedYear)){
            $scope.currentDate = moment(clickedYear.number, CONTROL_FORMAT_YEAR);
            $scope.changeCalendarMode('month', true);
        } else {
            var totalOfYearsToChange = isYearInLastDecade(clickedYear) ? -1 : +1;
            changeCurrentYearShownOnCalendar(totalOfYearsToChange);
        }
    };

    function callOnYearClickEvents(){
        var yearClickEventFunctions = ZhIconCalendarEvents.getRegisteredFunctionsForYearClick();
        callEventFunctions(yearClickEventFunctions);
    }

    function isYearInCurrentDecade(year){
        return (!year.lastDecade && !year.nextDecade);
    }

    function isYearBetweenMinAndMaxDate(year){
        var beginYear = moment(year.number, CONTROL_FORMAT_YEAR).startOf('year');
        var endYear = moment(year.number, CONTROL_FORMAT_YEAR).endOf('year');
        return (isDateAfterMinAllowed(beginYear) && isDateBeforeMaxAllowed(endYear));
    }

    function isYearInLastDecade(year){
        return !!year.lastDecade;
    }

    function changeCurrentYearShownOnCalendar(totalOfYearsToChange){
        totalOfYearsToChange = totalOfYearsToChange || +1;
        $scope.currentDate.add(totalOfYearsToChange * 10, 'year');
    }

	/**
	 * General codes
	 **/
	function callEventFunctions(functions){
        functions.forEach(function(functionName){
            if(calendarWidget[functionName]){
                calendarWidget[functionName]();
            }
        });
    }

	function isDateAfterMinAllowed(date){
	    var isAfter = true;
	    if($scope.momentMinDate.isValid()){
	        isAfter = date.isAfter($scope.momentMinDate);
	    }
	    return isAfter;
	}

	function isDateBeforeMaxAllowed(date){
	    var isBefore = true;
	    if($scope.momentMaxDate.isValid()){
	        isBefore = date.isBefore($scope.momentMaxDate);
	    }
	    return isBefore;
	}

    function isDayInLastMonth(day){
        return day.lastMonth;
    }

    function changeCurrentMonthShownOnCalendar(totalOfMonthsToChange){
        totalOfMonthsToChange = totalOfMonthsToChange || +1;
        $scope.currentDate.add(totalOfMonthsToChange, 'month');
    }

    /**
     * External functions
     **/
    this.isThereEventsOnDay = function(day){
        return day.events.length > 0;
    };

    this.getIconsFromDay = function(day){
	 	var iconProperty = calendarWidget.iconProperty;
	 	var icons = [];
	 	if(iconProperty){
	 		icons = day.events.map(function(item){
	 			return item[iconProperty];
	 		});

	 		icons = icons.filter(function(item, position){
	 			return icons.indexOf(item) === position;
	 		});
	 	}

	 	return icons;
	};

	this.isDaySelected = function(day){
	    return day === lastDayClicked;
	};
};

ZeedhiController.controller('ZhIconCalendarController', ZhIconCalendarController);

// FILE: src/controllers/ZhScheduler.js
var ZhSchedulerController = (function(_super) {
	__extends(ZhSchedulerController1, _super);

    function ZhSchedulerController1($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query, CrosstabService,
    $filter, maskEngine, ZhUtil, $window, ApplicationContext, ZhFeatures, eventAggregator, DecodeValueFieldService, ZhIconCalendarEvents, ZhCardsEvents, metaDataFactory) {
        var _ref = ZhSchedulerController.__super__.constructor.apply(this, arguments);

        var schedulerWidget = $scope.widget;

        var calendarWidget;
        var calendarWidgetMetaData;

        var cardsWidget;
        var cardsWidgetMetaData;

        var backToCalendarActionMetaData;

        var currentSevenDaysPeriod = [];

        (function initialize() {
            schedulerWidget.activate();

            createCalendarWidgetMetaData();
            factoryCalendarWidget();

            createCardsWidgetMetaData();
            factoryCardsWidget();

            createBackToCalendarActionMetaData();

            addUsefulFunctionsToSchedulerWidget();
        })();

        /********************************************
         * Internal functions
         ********************************************/
        function createCalendarWidgetMetaData() {
            calendarWidgetMetaData = {
                "container": getSchedulerContainer(),
                "id": "ZhSchedulerCalendar",
                "name": "ZH_SCHEDULER_CALENDAR",
                "template": "zh-scheduler#widget/calendar.html",
                "floatingControl": false,
                "dateFormat": schedulerWidget.dateFormat || 'YYYY-MM-DD',
                "dateField": schedulerWidget.dateField,
                "iconProperty": schedulerWidget.iconProperty,
                "dataSource": {
                    "rest": false,
                    "localStorage": false,
                    "data": schedulerWidget.dataSource.data
                },
                "events": getCalendarEvents()
            };
        }

        function removeActionBackToCalendarFromScheduler() {
            templateManager.container.actions.forEach(function(widget, index) {
                if (widget.name === 'ZH_SCHEDULER_BACK_TO_CALENDAR') {
                    templateManager.container.actions.splice(index, 1);
                }
            });
        }

        function hideCalendar() {
            $scope.showCalendar();
            removeActionBackToCalendarFromScheduler();
        }

        function getCalendarEvents() {
            var calendarEvents = getOnlyCalendarEventsFromScheduler();
            calendarEvents = addDayClickEventForReloadEvents(calendarEvents);
            calendarEvents = addOnEnterEventForGetTodayEvents(calendarEvents);
            return calendarEvents;
        }

        function getOnlyCalendarEventsFromScheduler() {
            var calendarEvents = ['WidgetOnDayClick', 'WidgetOnMonthClick', 'WidgetOnYearClick',
                'WidgetOnChangeMonth', 'WidgetOnChangeYear', 'WidgetOnChangeDecade'
            ];
            return schedulerWidget.events.filter(function(event) {
                return calendarEvents.some(function(calendarEvent) {
                    return event.name === calendarEvent;
                });
            });
        }

        function addDayClickEventForReloadEvents(events) {
            events.push({
                "name": "WidgetOnDayClick",
                "code": onDayClick
            });
            return events;
        }

        function onDayClick() {
            if ($scope.isProbablyTabletDevice()) {
                $scope.hideCalendar();
                addActionBackToCalendarOnScheduler();
            }

            prepareCardsWidgetToBeShown();
        }

        function prepareCardsWidgetToBeShown() {
            buildSevenDaysPeriod();
            filterEventsDataByClickedDay();
        }

        function addOnEnterEventForGetTodayEvents(events) {
            events.push({
                "name": "WidgetOnEnter",
                "code": onEnterCalendar
            });
            return events;
        }

        function addActionBackToCalendarOnScheduler() {
            templateManager.container.actions.push(Util.extend(new Action(), backToCalendarActionMetaData));
        }

        function buildSevenDaysPeriod() {
            var currentDay = calendarWidget.currentRow;
            var dateFormat = schedulerWidget.dateFormat;
            currentSevenDaysPeriod = [];
            for (var i = -3; i < 4; i++) {
                var day = moment(currentDay, dateFormat).add(i, 'days').format(dateFormat);
                currentSevenDaysPeriod.push(day);
            }
        }

        function onEnterCalendar() {
            filterEventsDataByClickedDay();
        }

        function filterEventsDataByClickedDay() {
            var lastDayClicked = moment(calendarWidget.currentRow, schedulerWidget.dateFormat);
            cardsWidget.dataSource.data = schedulerWidget.dataSource.data.filter(function(data) {
                var dayToCompare = moment(data[schedulerWidget.dateField], schedulerWidget.dateFormat);
                return dayToCompare.isSame(lastDayClicked,'day');
            });
        }

        function createCardsWidgetMetaData() {
            cardsWidgetMetaData = {
                "container": getSchedulerContainer(),
                "id": "ZhSchedulerCards",
                "name": "ZH_SCHEDULER_CARDS",
                "template": "zh-scheduler#widget/cards.html",
                "floatingControl": false,
                "color": schedulerWidget.color,
                "fields": schedulerWidget.fields,
                "noMatchText": schedulerWidget.noEventsText || "No events were found on this day.",
                "events": getOnlyCardsEvents()
            };
        }

        function getOnlyCardsEvents() {
            var cardsEvents = ['WidgetOnEventClick'];
            return schedulerWidget.events.filter(function(event) {
                return cardsEvents.some(function(cardEvent) {
                    return event.name === cardEvent;
                });
            });
        }

        function getSchedulerContainer() {
            return schedulerWidget.container;
        }

        function factoryCalendarWidget() {
            calendarWidget = metaDataFactory.widgetFactory(calendarWidgetMetaData);
        }

        function factoryCardsWidget() {
            cardsWidget = metaDataFactory.widgetFactory(cardsWidgetMetaData);
        }

        function createBackToCalendarActionMetaData() {
            backToCalendarActionMetaData = {
                "name": "ZH_SCHEDULER_BACK_TO_CALENDAR",
                "label": "Calendar",
                "showAsAction": "back",
                "isVisible": true,
                "widget": schedulerWidget,
                "events": [{
                    "name": "ActionEvent",
                    "code": hideCalendar
                }]
            };
        }

        function addUsefulFunctionsToSchedulerWidget() {
            schedulerWidget.getCalendarWidget = function() {
                return calendarWidget;
            };

            schedulerWidget.getEventsWidget = function() {
                return cardsWidget;
            };

            schedulerWidget.reloadSchedulerWidget = function() {
                schedulerWidget.reload();
                templateManager.updateTemplate();
            };

        }

        /********************************************
         * Functions used by template
         ********************************************/

        this.getCalendarWidgetTemplate = function() {
            return templateManager.getTemplateURL('zh-scheduler#widget/calendar.html');
        };

        this.getCardsWidgetTemplate = function() {
            return templateManager.getTemplateURL('zh-scheduler#widget/cards.html');
        };

        this.getSevenDaysPeriodTemplate = function() {
            return templateManager.getTemplateURL('zh-scheduler#widget/seven-days-period.html');
        };

        this.getCalendarWidget = function() {
            return [calendarWidget];
        };

        this.getCardsWidget = function() {
            return [cardsWidget];
        };

        this.getDaysFromSevenDaysPeriod = function() {
            var dateFormat = schedulerWidget.dateFormat;
            var onlyDays = [];
            currentSevenDaysPeriod.forEach(function(day) {
                day = moment(day, dateFormat).format(dateFormat);
                onlyDays.push(day);
            });
            return onlyDays;
        };

        this.getOnlyWeekDayNamesFromSevenDaysPeriod = function() {
            var weekDayNames = moment.weekdaysShort();
            var dateFormat = schedulerWidget.dateFormat;
            var onlyWeekDayNames = [];
            currentSevenDaysPeriod.forEach(function(day) {
                var dayIndex = moment(day, dateFormat).weekday();
                onlyWeekDayNames.push(weekDayNames[dayIndex]);
            });
            return onlyWeekDayNames;
        };

        this.isDaySameAsSelected = function(day) {
            var dateFormat = schedulerWidget.dateFormat;
            var selectedDay = moment(calendarWidget.currentRow, dateFormat).format(dateFormat);
            return day === selectedDay;
        };

        this.onCurrentWeekDayClick = function(day) {
            calendarWidget.currentRow = day;
            prepareCardsWidgetToBeShown();
        };

        this.getOnlyDayOfMonth = function(day) {
            var dateFormat = schedulerWidget.dateFormat;
            return moment(day, dateFormat).date().toString();
        };

        this.shouldShowEvents = function() {
            return !$scope.calendarVisibilityStatus || !$scope.isProbablyTabletDevice();
        };

        /********************************************
         * Angular Watches
         ********************************************/
        $scope.$watchCollection(function(){
            return schedulerWidget.dataSource.data;
        }, function(newValue){
            filterEventsDataByClickedDay();
            calendarWidget.dataSource.data = newValue;
        });

    }

    return ZhSchedulerController1;
})(ListController);

ZeedhiController.controller('ZhSchedulerController', ZhSchedulerController);

// FILE: src/directives/ZhIconCalendar.js
(function(){

	var ZeedhiDirectives = angular.module('ZeedhiDirectives');
	ZeedhiDirectives.directive('zhIconCalendar', zhIconCalendar);

	function zhIconCalendar(ZhUtil, ScreenService){

		return {
			restrict: 'A',
			link: function($scope, element){

				var controlFormat = "YYYY-MM-DD";
				var controlFormatMonth = "YYYY-MM";
				var controlFormatYear = "YYYY";

				var MODE_DAY = "day";
				var MODE_MONTH = "month";
				var MODE_YEAR = "year";

				var UNIT_DAY = "day";
				var UNIT_WEEK = "week";
				var UNIT_MONTH = "month";
				var UNIT_YEAR = "year";

				var widget = $scope.widget;
				widget.selectedDates = [];

				$scope.calendarMode = widget.calendarMode || MODE_DAY;
				$scope.today = moment().startOf(UNIT_DAY);
				$scope.widget.currentRow = $scope.today.format($scope.widget.dateFormat);
				$scope.dateFormat = $scope.widget.dateFormat || "L";

				if(widget.field){
					if (widget.field.currentDate) {
						$scope.currentDate = moment(widget.field.currentDate, widget.field.dateFormat);
					} else if (widget.field.defaultValue) {
						$scope.currentDate = moment(ZhUtil.evaluate(widget.field.defaultValue), $scope.dateFormat).startOf(UNIT_MONTH).startOf(UNIT_DAY);
					}
				} else {
					if (widget.currentDate) {
						$scope.currentDate = moment(widget.currentDate, $scope.dateFormat).startOf(UNIT_MONTH).startOf(UNIT_DAY);
					}
				}

				if (!$scope.currentDate || !$scope.currentDate.isValid()) {
					$scope.currentDate = moment().startOf(UNIT_MONTH).startOf(UNIT_DAY);
				}

				$scope.todayText = $scope.widget.todayText || ScreenService.i18n("Today");
				$scope.weekDays = $scope.widget.weekDays || moment.weekdaysShort();
				$scope.monthNames = $scope.widget.monthNames || moment.months();

				$scope.maxDate = "";
				$scope.momentMaxDate = "";

				$scope.minDate = "";
				$scope.momentMinDate = "";

				widget.setDateAsSelected = function(dateStr, dateFormat) {
					widget.selectedDates.push( moment(dateStr, dateFormat || widget.dateFormat || controlFormat).format(controlFormat) );
				};

				function updateMaxAndMinDates() {
					$scope.maxDate = $scope.widget.maxDate === "today" ? moment().format($scope.dateFormat) : $scope.widget.maxDate;
					$scope.momentMaxDate = moment($scope.maxDate, $scope.dateFormat);

					$scope.minDate = $scope.widget.minDate === "today" ? moment().format($scope.dateFormat) : $scope.widget.minDate;
					$scope.momentMinDate = moment($scope.minDate, $scope.dateFormat);
				}
				updateMaxAndMinDates();


				$scope.isWeekdayToday = function(weekdayIndex){
					var todayWeekday = moment().day();
					return weekdayIndex === todayWeekday;
				};


				// Auxiliar functions

				function isBlockedDate(date) {
					var dayMoment = moment(date, $scope.dateFormat);
					return (
						($scope.momentMinDate.isValid() && dayMoment.isBefore($scope.momentMinDate)) ||
						($scope.momentMaxDate.isValid() && dayMoment.isAfter($scope.momentMaxDate))
					);
				}

				function isBlockedMonth(monthDate) {
					var monthMomentBegining = moment(monthDate, controlFormatMonth).startOf(UNIT_MONTH);
					var monthMomentEnd = moment(monthDate, controlFormatMonth).endOf(UNIT_MONTH);
					return (
						($scope.momentMaxDate.isValid() && monthMomentBegining.isAfter($scope.momentMaxDate)) ||
						($scope.momentMinDate.isValid() && monthMomentEnd.isBefore($scope.momentMinDate))
					);
				}

				function isBlockedYear(yearNumber) {
					var yearMomentBegining = moment(yearNumber, controlFormatYear).startOf(UNIT_YEAR);
					var yearMomentEnd = moment(yearNumber, controlFormatYear).endOf(UNIT_YEAR);
					return (
						($scope.momentMaxDate.isValid() && yearMomentBegining.isAfter($scope.momentMaxDate)) ||
						($scope.momentMinDate.isValid() && yearMomentEnd.isBefore($scope.momentMinDate))
					);
				}

				function isSelectedDate(date) {
					return widget.selectedDates.indexOf(date) != -1;
				}

				$scope.isBlockedDate = isBlockedDate;
				$scope.isBlockedMonth = isBlockedMonth;
				$scope.isBlockedYear = isBlockedYear;

				function changeCalendarMode(mode, changeToParameterMode) {
					var newMode = mode;

					if(!changeToParameterMode) {
						switch (mode) {
							case MODE_DAY:
								newMode = MODE_MONTH;
								break;
							case MODE_MONTH:
								newMode = MODE_YEAR;
								break;
						}
					}

					$scope.calendarMode = newMode;
				}

				$scope.changeCalendarMode = changeCalendarMode;

				$scope.getCalendarHeading = function(mode) {
					var heading = "";

					switch (mode) {
						case MODE_YEAR:
							heading = $scope.yearRows ? $scope.yearRows[0].years[0].number + " - " + $scope.yearRows[2].years[3].number : "";
							break;
						case MODE_MONTH:
							heading = $scope.currentDate.format(controlFormatYear);
							break;
						case MODE_DAY:
							heading = $scope.currentDate.format("MMMM YYYY");
							break;
					}

					return heading;
				};

				// Navigation functions
				$scope.navigateCalendar = function(value) {
					switch ($scope.calendarMode) {
						case MODE_YEAR:
							navigateYearMode(value);
							break;
						case MODE_MONTH:
							navigateMonthMode(value);
							break;
						case MODE_DAY:
							navigateDayMode(value);
							break;
					}
				};

				function navigateYearMode(value) {
					$scope.currentDate.add(value * 10, UNIT_YEAR);
				}

				function navigateMonthMode(value) {
					$scope.currentDate.add(value, UNIT_YEAR);
				}

				function navigateDayMode(value) {
					$scope.currentDate.add(value, UNIT_MONTH);
				}




				// Calendar rendering functions
				function buildCalendar(mode) {
					switch (mode) {
						case MODE_YEAR:
							buildYearCalendar();
							break;
						case MODE_MONTH:
							buildMonthCalendar();
							break;
						case MODE_DAY:
							buildDayCalendar();
							break;
					}
				}

				function buildDayCalendar() {
					// Clones the current date to prevent changing it
					var current = $scope.currentDate.clone().startOf(UNIT_WEEK).day(0);

					// Checks if the first day of the month is also the first day of its week so we should put it on the second row of the calendar
					if (current.format(controlFormat) === $scope.currentDate.clone().format(controlFormat)) {
						current.subtract(7, UNIT_DAY);
					}

					// Iterates through the datasource seeking for this month events and builds an object grouping them
					var monthEvents = {};
					if ($scope.widget.dataSource && $scope.widget.dataSource.data) {
						$scope.widget.dataSource.data
							.filter(function(item) {
								var month = moment(item[$scope.widget.dateField], $scope.dateFormat).get(UNIT_MONTH);
								var currMonth = current.get(UNIT_MONTH);
								var isPreviousMonth = month === currMonth;
								var isCurrentMonth = month === ((currMonth + 1) % 12);
								var isNextMonth = month === ((currMonth + 2) % 12);
								return isPreviousMonth || isCurrentMonth || isNextMonth;
							})
							.forEach(function(item) {
								var itemTime = moment(item[$scope.widget.dateField], $scope.dateFormat).startOf(UNIT_DAY).format(controlFormat);

								if (!monthEvents[itemTime]) {
									monthEvents[itemTime] = [];
								}

								monthEvents[itemTime].push(item);
							});
					}

					// Start creating week
					$scope.weeks = [];
					for (var i = 0; i < 6; i++) {
						var week = {
							days: []
						};
						for (var j = 0; j < 7; j++) {
							week.days.push({
								day: current.date(),
								date: current.format($scope.dateFormat),
								today: current.format(controlFormat) === $scope.today.format(controlFormat),
								lastMonth: (current.month() < $scope.currentDate.month() && current.year() == $scope.currentDate.year()) || (current.month() > $scope.currentDate.month() && current.year() < $scope.currentDate.year()),
								nextMonth: (current.month() > $scope.currentDate.month() && current.year() == $scope.currentDate.year()) || (current.month() < $scope.currentDate.month() && current.year() > $scope.currentDate.year()),
								events: monthEvents[current.format(controlFormat)] || [],
								selected: isSelectedDate(current.format(controlFormat))
							});
							current.add(1, UNIT_DAY);
						}
						$scope.weeks.push(week);
					}
				}

				function buildMonthCalendar() {
					var auxiliarRow = [];
					$scope.monthRows = [];

					$scope.currentDate.startOf(UNIT_YEAR);
					var year = $scope.currentDate.format(controlFormatYear);

					var shortMonthNames = moment.monthsShort();
					for(var i=0; i<12; i++) {
						auxiliarRow.push({
							date: moment("01/" + (i+1) + "/" + year, "DD/MM/YYYY").format(controlFormatMonth),
							selected: false,
							name: shortMonthNames[i]
						});

						if(auxiliarRow.length == 4) {
							$scope.monthRows.push({ "months": auxiliarRow });
							auxiliarRow = [];
						}
					}
				}

				function buildYearCalendar() {
					var auxiliarRow = [];
					$scope.yearRows = [];

					var currentYear = parseInt($scope.currentDate.format(controlFormatYear));
					var gridInitialYear = Math.floor(currentYear / 10) * 10;

					$scope.currentDate = moment(gridInitialYear, controlFormatYear);
					for(var i=-1; i<11; i++) {
						auxiliarRow.push({
							number: moment("01/01/" + (gridInitialYear + i), "DD/MM/YYYY").format(controlFormatYear),
							selected: false,
							lastDecade: i == -1,
							nextDecade: i == 10,
						});

						if(auxiliarRow.length == 4) {
							$scope.yearRows.push({ "years": auxiliarRow });
							auxiliarRow = [];
						}
					}
				}

				// Event triggering functions
				function triggerChangeEvents(currentMoment, oldMoment) {
					var monthBeforeChange = oldMoment.month();
					var monthAfterChange = currentMoment.month();

					var yearBeforeChange = oldMoment.year();
					var yearAfterChange = currentMoment.year();

					var decadeBeforeChange = Math.floor(oldMoment.year() / 10) * 10;
					var decadeAfterChange = Math.floor(currentMoment.year() / 10) * 10;

					if (monthBeforeChange != monthAfterChange) {
						triggerMonthChangeEvent();
					}

					if (yearBeforeChange != yearAfterChange) {
						triggerYearChangeEvent();
					}

					if (decadeBeforeChange != decadeAfterChange) {
						triggerDecadeChangeEvent();
					}
				}

				function triggerMonthChangeEvent() {
					if(widget.onChangeMonth){
						widget.onChangeMonth();
					}
				}

				function triggerYearChangeEvent() {
					if(widget.onChangeYear){
						widget.onChangeYear();
					}
				}

				function triggerDecadeChangeEvent() {
					if(widget.onChangeDecade){
						widget.onChangeDecade();
					}
				}



				// Watches
				$scope.$watch("calendarMode", function() {
					buildCalendar($scope.calendarMode);
				});

				$scope.$watch(
					function() {
						return $scope.currentDate._d.getTime();
					},
					function(newTimestamp, oldTimestamp) {
						var currentMoment = moment(new Date(newTimestamp));
						var oldMoment = moment(new Date(oldTimestamp));

						buildCalendar($scope.calendarMode);
						triggerChangeEvents(currentMoment, oldMoment);
					}
				);

				$scope.$watchCollection("widget.selectedDates",
					function() {
						buildCalendar($scope.calendarMode);
					}
				);

				$scope.$watchCollection(function() {
					if ($scope.widget.dataSource) {
						return $scope.widget.dataSource.data;
					}
				}, function() {
					buildCalendar($scope.calendarMode);
				});

				$scope.$watch(
					function() {
						return $scope.widget.minDate;
					},
					function() {
						if($scope.widget.minDate){
							updateMaxAndMinDates();
						}
					}
				);

				$scope.$watch(
					function() {
						return $scope.widget.maxDate;
					},
					function() {
						if($scope.widget.maxDate){
							updateMaxAndMinDates();
						}
					}
				);

				$scope.widget.getYear = function(){
					return $scope.currentDate.get('year');
				};

				$scope.widget.getMonthName = function() {
					return  $scope.monthNames[$scope.currentDate.get('month')];
				};

				$scope.widget.getMonth = function(){
					return $scope.currentDate.get('month');
				};

			}

		};
	}

})();

// FILE: src/directives/ZhScheduler.js
(function(){
    
    var ZeedhiDirectives = angular.module('ZeedhiDirectives');
    ZeedhiDirectives.directive('zhScheduler', zhScheduler);
    
    function zhScheduler(){
        
        var scope;
        var element;
        var attributes;
        var controller;
        
        var linkFunction = function(theScope, theElement, theAttributes, theController){
            scope = theScope;
            element = theElement;
            attributes = theAttributes;
            controller = theController;
            
            scope.calendarVisibilityStatus = true;
            
            scope.hideCalendar = function(){
                var domCalendar = getDomCalendarByParentElement(element);
                scope.setCalendarVisibilityStatus(false);
                domCalendar.slideUp();
            };
            
            scope.showCalendar = function(){
                var domCalendar = getDomCalendarByParentElement(element);
                scope.setCalendarVisibilityStatus(true);
                domCalendar.slideDown();
            };
            
            scope.isProbablyTabletDevice = function(){
                return $(document).width() < 769;    
            };
            
            scope.setCalendarVisibilityStatus = function(status){
                scope.calendarVisibilityStatus = status;
            };
            
            function getDomCalendarByParentElement(element){
                return element.find('section.zh-icon-calendar');
            }
        
        };
        
        return {
            restrict: 'A',
            link: linkFunction
        };
        
    }
})();